@@ -6,8 +6,8 @@ Event monitor plugin.
 @copyright: Copyright (c) 2007 IBM Corporation
 @license: BSD
 
-All rights reserved. This program and the accompanying materials are made 
-available under the terms of the BSD which accompanies this distribution, and 
+All rights reserved. This program and the accompanying materials are made
+available under the terms of the BSD which accompanies this distribution, and
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import gi
@@ -24,7 +24,7 @@ from accerciser.plugin import ViewportPlugin
 from accerciser.i18n import _, N_, DOMAIN
 from accerciser import node
 
-UI_FILE = os.path.join(os.path.dirname(__file__), 
+UI_FILE = os.path.join(os.path.dirname(__file__),
                        'event_monitor.ui')
 
 class EventMonitor(ViewportPlugin):
@@ -32,7 +32,7 @@ class EventMonitor(ViewportPlugin):
   Class for the monitor viewer.
 
   @ivar source_filter: Determines what events from what sources could be shown.
-  Either source_app and source_acc for selected applications and accessibles 
+  Either source_app and source_acc for selected applications and accessibles
   respectively. Or everything.
   @type source_filter: string
   @ivar main_xml: The main event monitor gtkbuilder file.
@@ -130,13 +130,13 @@ class EventMonitor(ViewportPlugin):
     Initialize text view in monitor plugin.
     '''
     self.textview_monitor = self.main_xml.get_object('textview_monitor')
-    
+
     self.monitor_buffer = self.textview_monitor.get_buffer()
     self.monitor_mark = \
-        self.monitor_buffer.create_mark('scroll_mark', 
+        self.monitor_buffer.create_mark('scroll_mark',
                                         self.monitor_buffer.get_end_iter(),
                                         False)
-    self.monitor_buffer.create_mark('mark_last_log', 
+    self.monitor_buffer.create_mark('mark_last_log',
                                     self.monitor_buffer.get_end_iter(),
                                     True)
     self.monitor_buffer.create_tag('last_log', weight=700)
@@ -144,7 +144,7 @@ class EventMonitor(ViewportPlugin):
   def _appendChildren(self, parent_iter, parent, level, events):
     '''
     Append child events to a parent event's iter.
-    
+
     @param parent_iter: The tree iter of the parent.
     @type parent_iter: gtk.TreeIter
     @param parent: The parent event.
@@ -153,22 +153,22 @@ class EventMonitor(ViewportPlugin):
     @type level: integer
     @param events: A list of children
     @type events: list
-    
+
     @return: Return false to that this won't be called again in the mainloop.
     @rtype: boolean
     '''
     for event in events:
       if event.count(':') == level and event.startswith(parent):
-        iter = self.events_model.append(parent_iter, 
+        iter = self.events_model.append(parent_iter,
                                         [event.split(':')[-1],
                                          event, False, False])
         GLib.idle_add(self._appendChildren, iter, event, level + 1, events)
     return False
-        
+
   def _onToggled(self, renderer_toggle, path):
     '''
     Callback for toggled events in the treeview.
-    
+
     @param renderer_toggle: The toggle cell renderer.
     @type renderer_toggle: gtk.CellRendererToggle
     @param path: The path of the toggled node.
@@ -181,25 +181,25 @@ class EventMonitor(ViewportPlugin):
 
   def _resetClient(self):
     '''
-    De-registers the client from the currently monitored events. 
-    If the monitor is still enabled it get's a list of enabled events 
+    De-registers the client from the currently monitored events.
+    If the monitor is still enabled it get's a list of enabled events
     and re-registers the client.
     '''
-    pyatspi.Registry.deregisterEventListener(self._handleAccEvent, 
+    pyatspi.Registry.deregisterEventListener(self._handleAccEvent,
                                              *self.listen_list)
     self.listen_list = self._getEnabledEvents(self.events_model.get_iter_first())
     if self.monitor_toggle.get_active():
-      pyatspi.Registry.registerEventListener(self._handleAccEvent, 
+      pyatspi.Registry.registerEventListener(self._handleAccEvent,
                                              *self.listen_list)
 
   def _getEnabledEvents(self, iter):
     '''
-    Recursively walks through the events model and collects all enabled 
+    Recursively walks through the events model and collects all enabled
     events in a list.
-    
+
     @param iter: Iter of root node to check under.
     @type iter: gtk.TreeIter
-    
+
     @return: A list of enabled events.
     @rtype: list
     '''
@@ -217,10 +217,10 @@ class EventMonitor(ViewportPlugin):
 
   def _iterToggle(self, iter, val):
     '''
-    Toggle the given node. If the node has children toggle them accordingly 
+    Toggle the given node. If the node has children toggle them accordingly
     too. Toggle all anchester nodes too, either true, false or inconsistent,
     sepending on the value of their children.
-    
+
     @param iter: Iter of node to toggle.
     @type iter: gtk.TreeIter
     @param val: Toggle value.
@@ -232,16 +232,16 @@ class EventMonitor(ViewportPlugin):
     parent = self.events_model.iter_parent(iter)
     while parent:
       is_consistent = self._descendantsConsistent(parent)
-      self.events_model.set_value(parent, 
+      self.events_model.set_value(parent,
                                   self.COL_INCONSISTENT,
                                   not is_consistent)
       self.events_model.set_value(parent, self.COL_TOGGLE, val)
       parent = self.events_model.iter_parent(parent)
-  
+
   def _setAllDescendants(self, iter, val):
     '''
     Set all descendants of a given node to a certain toggle value.
-    
+
     @param iter: Parent node's iter.
     @type iter: gtk.TreeIter
     @param val: Toggle value.
@@ -252,14 +252,14 @@ class EventMonitor(ViewportPlugin):
       self.events_model.set_value(child, self.COL_TOGGLE, val)
       self._setAllDescendants(child, val)
       child = self.events_model.iter_next(child)
-  
+
   def _descendantsConsistent(self, iter):
     '''
     Determine if all of a node's descendants are consistently toggled.
-    
+
     @param iter: Parent node's iter.
     @type iter: gtk.TreeIter
-    
+
     @return: True if descendants nodes are consistent.
     @rtype: boolean
     '''
@@ -277,7 +277,7 @@ class EventMonitor(ViewportPlugin):
   def _onSelectAll(self, button):
     '''
     Callback for "select all" button. Select all event types.
-    
+
     @param button: Button that was clicked
     @type button: gtk.Button
     '''
@@ -290,7 +290,7 @@ class EventMonitor(ViewportPlugin):
   def _onClearSelection(self, button):
     '''
     Callback for "clear selection" button. Clear all selected events.
-    
+
     @param button: Button that was clicked.
     @type button: gtk.Button
     '''
@@ -303,13 +303,13 @@ class EventMonitor(ViewportPlugin):
   def _logEvent(self, event):
     '''
     Log the given event.
-    
+
     @param event: The event to log.
     @type event: Accessibility.Event
     '''
     iter = self.monitor_buffer.get_iter_at_mark(self.monitor_mark)
     self.monitor_buffer.move_mark_by_name(
-      'mark_last_log', 
+      'mark_last_log',
       self.monitor_buffer.get_iter_at_mark(self.monitor_mark))
     self._insertEventIntoBuffer(event)
     self.textview_monitor.scroll_mark_onscreen(self.monitor_mark)
@@ -318,13 +318,14 @@ class EventMonitor(ViewportPlugin):
     '''
     Inserts given event in to text buffer. Creates hyperlinks for
     the events context accessibles.
-    
+
     @param event: The at-spi event we are inserting.
     @type event: Accessibility.Event
     '''
-    self._writeText('%s(%s, %s, %s)\n\tsource: ' % \
-                      (event.type, event.detail1, 
-                       event.detail2, event.any_data))
+    ts = int((os.times()[-1] * 10 ) % 1000)
+    self._writeText('%02.1f %s(%s, %s, %s)\n\tsource: ' % \
+                      (ts / 10, event.type, event.detail1,
+                        event.detail2, event.any_data))
     hyperlink = self._createHyperlink(event.source)
     self._writeText(str(event.source), hyperlink)
     self._writeText('\n\tapplication: ')
@@ -352,7 +353,7 @@ class EventMonitor(ViewportPlugin):
     '''
     Convinience function for inserting text in to the text buffer.
     If tags are provided they are applied to the inserted text.
-    
+
     @param text: Text to insert
     @type text: string
     @param *tags: List of optional tags to insert with text
@@ -371,16 +372,15 @@ class EventMonitor(ViewportPlugin):
     '''
     Create a hyperlink tag for a given accessible. When the link is clicked
     the accessible is selected in the main program.
-    
+
     @param acc: The accessible to create the tag for.
     @type acc: Accessibility.Accessible
-    
+
     @return: The new hyperlink tag
     @rtype: gtk.TextTag
     '''
     hyperlink = self.monitor_buffer.create_tag(
-      None, 
-      foreground='blue',
+      None,
       underline=Pango.Underline.SINGLE)
     hyperlink.connect('event', self._onLinkClicked)
     setattr(hyperlink, 'acc', acc)
@@ -390,7 +390,7 @@ class EventMonitor(ViewportPlugin):
   def _onLinkClicked(self, tag, widget, event, iter):
     '''
     Callback for clicked link. Select links accessible in main application.
-    
+
     @param tag: Tag that was clicked.
     @type tag: gtk.TextTag
     @param widget: The widget that received event.
@@ -401,20 +401,20 @@ class EventMonitor(ViewportPlugin):
     @type iter: gtk.TextIter
     '''
     if event.type == gdk.EventType.BUTTON_RELEASE and \
-           event.button == 1 and not self.monitor_buffer.get_has_selection():
+           event.button.button == 1 and not self.monitor_buffer.get_has_selection():
       self.node.update(getattr(tag, 'acc'))
 
   def _onLinkKeyPress(self, textview, event):
     '''
-    Callback for a keypress in the text view. If the keypress is enter or 
+    Callback for a keypress in the text view. If the keypress is enter or
     space, and the cursor is above a link, follow it.
-    
+
     @param textview: Textview that was pressed.
     @type textview: gtk.TextView
     @param event: Event object.
     @type event: gdk.Event
     '''
-    if event.keyval in (gdk.KEY_Return, 
+    if event.keyval in (gdk.KEY_Return,
                         gdk.KEY_KP_Enter,
                         gdk.KEY_space):
       buffer = textview.get_buffer()
@@ -429,12 +429,12 @@ class EventMonitor(ViewportPlugin):
   def _onLinkMotion(self, textview, event):
     '''
     Change mouse cursor shape when hovering over a link.
-    
+
     @param textview: Monitors text view.
     @type textview: gtk.TextView
     @param event: Event object
     @type event: gdk.Event
-    
+
     @return: Return False so event continues in callback chain.
     @rtype: boolean
     '''
@@ -452,13 +452,12 @@ class EventMonitor(ViewportPlugin):
           break
     window = textview.get_window(gtk.TextWindowType.TEXT)
     window.set_cursor(cursor)
-    window.get_pointer()
     return False
 
   def _handleAccEvent(self, event):
     '''
     Main at-spi event client. If event passes filtering requirements, log it.
-    
+
     @param event: The at-spi event recieved.
     @type event: Accessibility.Event
     '''
@@ -468,57 +467,56 @@ class EventMonitor(ViewportPlugin):
 
   def _onSave(self, button):
     '''
-    Callback for 'save' button clicked. Saves the buffer in to the given 
+    Callback for 'save' button clicked. Saves the buffer in to the given
     filename.
-    
+
     @param button: Button that was clicked.
     @type button: gtk.Button
     '''
-    save_dialog = gtk.FileChooserDialog(
-      'Save monitor output',
-      action=gtk.FileChooserAction.SAVE,
-      buttons=(gtk.ButtonsType.CANCEL, gtk.ResponseType.CANCEL,
-               gtk.ButtonsType.OK, gtk.ResponseType.OK))
+    save_dialog = gtk.FileChooserNative.new(
+      _('Save monitor output'),
+      self.get_toplevel(),
+      gtk.FileChooserAction.SAVE,
+      _('_OK'),
+      _('_Cancel'))
     save_dialog.set_do_overwrite_confirmation(True)
-    save_dialog.set_default_response(gtk.ResponseType.OK)
     response = save_dialog.run()
-    save_dialog.show_all()
-    if response == gtk.ResponseType.OK:
+    if response == gtk.ResponseType.ACCEPT:
       save_to = open(save_dialog.get_filename(), 'w')
       save_to.write(
         self.monitor_buffer.get_text(self.monitor_buffer.get_start_iter(),
-                                     self.monitor_buffer.get_end_iter()))
+                                     self.monitor_buffer.get_end_iter(), False))
       save_to.close()
     save_dialog.destroy()
-  
+
   def _onClear(self, button):
     '''
     Callback for 'clear' button. Clears monitor's text buffer.
-    
+
     @param button: Button that was clicked.
     @type button: gtk.Button
     '''
     self.monitor_buffer.set_text('')
 
-  
+
   def _onMonitorToggled(self, monitor_toggle):
     '''
     Callback for monitor toggle button. Activates or deactivates monitoring.
-    
+
     @param monitor_toggle: The toggle button that was toggled.
     @type monitor_toggle: gtk.ToggleButton
     '''
     if monitor_toggle.get_active():
-      pyatspi.Registry.registerEventListener(self._handleAccEvent, 
+      pyatspi.Registry.registerEventListener(self._handleAccEvent,
                                              *self.listen_list)
     else:
-      pyatspi.Registry.deregisterEventListener(self._handleAccEvent, 
+      pyatspi.Registry.deregisterEventListener(self._handleAccEvent,
                                                *self.listen_list)
 
   def _onSourceToggled(self, radio_button):
     '''
     Callback for radio button selection for choosing source filters.
-    
+
     @param radio_button: Radio button that was selected.
     @type radio_button: gtk.RadioButton
     '''
@@ -527,10 +525,10 @@ class EventMonitor(ViewportPlugin):
   def _eventFilter(self, event):
     '''
     Determine if an event's source should make the event filtered.
-    
+
     @param event: The given at-spi event.
     @type event: Accessibility.Event
-    
+
     @return: False if the event should be filtered.
     @rtype: boolean
     '''
@@ -543,7 +541,7 @@ class EventMonitor(ViewportPlugin):
       return event.source == self.acc
     else:
       return True
-  
+
   def _onHighlightEvent(self):
     '''
     A callback fom a global key binding. Makes the last event in the textview
